#ifndef CONTROLER_HPP
#define CONTROLER_HPP

#include "Game/Move.hpp"
#include "Utils/Listener.hpp"

template <typename GameBoard>
class Controler : public Listener<Move<typename GameBoard::Coords> >
{
    typedef typename GameBoard::Coords Coords;

private:

protected:
    int m_colorControlled;    /*!< Color associated to this controler */

public:
    /*! \brief Default constructor
     *  Will define an invalid color making this controller basically invalid
     */
    Controler():m_colorControlled(-1) {} // Put an invalid color

    // virtual ~Controler() = 0;
    /*! \brief Gets the move that the controler wants to play
     *  This function is generally waiting for the move from the controler.
     *  \return
     */
    virtual Move<Coords> getMove() =0;

    /*! \brief Defines the color used by the controler
     *  \param color
     */
    virtual void setColorControled(int color)
    {
        m_colorControlled = color;
    }
};

/*! \class Controler Controler.hpp Controlers/Controler.hpp
 *  \brief Base class for controlers
 *
 *  A controler is the input module to interact with the game.
 *  The controller should return next move (\c getMove()) of the game. This move is generally generated from
 *  user inputs or external inputs.
 *  Since a controler is like a game player, the controler has an associated color defined by setColorControled().
 *  \sa Move
 */

#endif
