#ifndef QTCONTROLER_HPP
#define QTCONTROLER_HPP

#include "Controler.hpp"
#include "Game/Boards/ChessBoard.hpp"

#include <QObject>
#include <QMutex>
#include <QSemaphore>

class QtControler : public QObject, public Controler<ChessBoard>
{
    Q_OBJECT

private:

    std::vector<Move<ChessBoard::Coords> > m_moves; /*! moves to play */

    /*! Synchonisation variables */
    QMutex m_mutex;
    QSemaphore m_semaphore;

    /*! Receive the move from a signal
     *  \param move
     */
    Q_SLOT void moveReceiver(Move<ChessBoard::Coords> move);

public:
    QtControler()
    {
    }

    ~QtControler()
    {
        m_mutex.unlock();
    }

    Move<ChessBoard::Coords> getMove();

    void notify(const Move<ChessBoard::Coords> & move) { (void)move; }

    void setColorControled(int color)
    {
        this->m_colorControlled = color;
    }
};

/*! \class QtControler QtControler.hpp Controlers/QtControler.hpp
 *  \brief Controler receiving the orders from the Qt UI
 *
 *  The move is received by a signal from the Qt UI. Since two different threads
 *  reach the class. Once the new move is received, the getMove() function
 *  will continue and return it.
 */

#endif
