#ifndef QTGNUCHESSCONTROLER_HPP
#define QTGNUCHESSCONTROLER_HPP

#include "Controler.hpp"
#include "Game/Boards/ChessBoard.hpp"

#include <QObject>
#include <QProcess>

#include <cassert>

class QtGNUChessControler : public QObject , public Controler<ChessBoard>
{
    Q_OBJECT

private:

    static const QString m_gnuChessPath;

    QProcess m_process;
    bool m_isStarted;

    void sendCommand(const std::string& command);

    Q_SLOT void errorOccured(QProcess::ProcessError);

public:
    QtGNUChessControler();
    ~QtGNUChessControler();

    /*! \brief Start the fork wrapping GNUChess
     *  The standard input and output from GNUChess are redirected to discuss with the parent
     *  GNUChess is started with -x option to make it easier to collect moves from it
     */
    void start();
    Move<ChessBoard::Coords> getMove();

    void notify(const Move<ChessBoard::Coords> & move);

    /*! 
     *  Since GNUChess basically does not know which color it is controlling,
     *  when \a color is white, we tell it to GNUChess
     */
    void setColorControled(int color)
    {
        assert(m_isStarted);

        m_colorControlled = color;
        if ( color == 0 )
        {
            sendCommand("white");
            sendCommand("go");
        }
    }

    static bool checkAvailability();
};

/*! \class GNUChessControler GNUChessControler.hpp Controlers/GNUChessControler.hpp
 *  \brief GNUChess interface to control the game
 *
 *  Interacts with GNUChess to play in the game.
 *  Thanks to the listener system, \c notify() function will send back the commands to GNUChess.
 *  Moves generated by GNUChess are collected with \c getMove(). The interaction between GNUChess and the Game is done
 *  by redirecting input and outputs from GNUChess fork.
 */

#endif
