#ifndef BOARD_HPP
#define BOARD_HPP

#include "Game/Piece.hpp"

template <int dimensions>
class Board
{
private:

public:
    static const int DIMENSIONS = dimensions;   /*!< Number of dimensions of the board (e.g.: 2 for 2D / 3 for 3D) */

    /*!
     */
    virtual ~Board() {}

    /*! Gets a piece from the board
        \param indexes the indexes for each dimensions to locate the piece
        \return the <a>Piece</a> (can be an invalid piece if there is no Piece here)
        \sa Piece
     */
    virtual const Piece& get(const int indexes[dimensions])const=0;

    /*! Gets the size of a precise dimension
     *  \param dimensionIndex index of the dimension requested
     *  \return
     */
    virtual int getDimensionLength(unsigned int dimensionIndex)const=0;
};

/*! \class Board Board.hpp Game/Boards/Board.hpp
 *  \brief Generic board where the game takes place
 *
 * This is the base class to declare a board. Technically, a board is just a place
 * where the pieces are. So, the board is giving simple function (\c get()) to get a piece to a precise location.
 * Moreover, some information about the size of the board can be requested using \c getDimensionLength()
 * \sa Piece
 */

#endif
