#ifndef BOARD2D_HPP
#define BOARD2D_HPP

#include "Board.hpp"

#include "Game/Move.hpp"
#include "Game/Piece.hpp"

#include <vector>

class Board2D : public Board<2>
{
public:
    typedef struct Coords
    {
        int column;
        int line;

        Coords():column(-1),line(-1) {}
        Coords(int column, int line):column(column),line(line) { }
        Coords(const int indexes[2]) { column = indexes[0] ; line = indexes[1]; }
        const int* get()const { return &column; }

        bool isValid()const { return column >= 0 && column < 8 && line >= 0 && line < 8; }

        bool operator!=(const Coords& c)
        {
            return column != c.column || line != c.line;
        }
    }Coords;

    static const Coords InvalidCoords;

protected:

    std::vector < std::vector < Piece > > m_board;

public:
    Board2D(unsigned int columnNumber, unsigned int lineNumber);

    void init(const std::vector<std::pair<Coords, Piece> >& startPieces);
    const Piece& get(const int indexes[2])const;
    const Piece& get(const Coords& coords)const;

    const std::vector<std::pair<Coords, Piece> > getPieces()const;

    /*! Moves pieces on board as described by \a move
     *  \param move
     */
    virtual void move(const Move<Coords>& move)=0;
};

#endif
