#include "ChessBoard.hpp"

#include "Game/Rules/ChessRules.hpp"

#include <cmath>
#include <cassert>

void ChessBoard::castling(const Move<Coords>& move)
{
    if ( move.dest().column == 2 ) // Big castling
    {
        // King
        m_board[2][move.start().line] = m_board[move.start().column][move.start().line];
        m_board[move.start().column][move.start().line] = Piece();

        // Move rook
        m_board[3][move.start().line] = m_board[0][move.start().line];
        m_board[0][move.start().line] = Piece();
    }
    else if (move.dest().column == 6) // Little castling
    {
        // Move king
        m_board[6][move.start().line] = m_board[move.start().column][move.start().line];
        m_board[move.start().column][move.start().line] = Piece();

        // Move rook
        m_board[5][move.start().line] = m_board[7][move.start().line];
        m_board[7][move.start().line] = Piece();
    }
    else
    {
        assert(false);
    }
}

void ChessBoard::move(const Move<Coords>& move)
{
    if ( move.isLegal() )
    {
        const Coords& source = move.start();
        const Coords& dest = move.dest();
        Piece sourcePiece = m_board[source.column][source.line];

        if ( sourcePiece.type() == ChessRules::PT_KING &&
             source.line == dest.line &&
             (source.line == 0 || source.line == 7) &&
             std::abs(static_cast<float>(source.column - dest.column)) == 2 )
        {
            if ( sourcePiece.hasProperty("hasMoved") == false ||
                 ( sourcePiece.hasProperty("hasMoved") == true && sourcePiece.getProperty("hasMoved") == false ) )
            {
                castling(move);
                m_board[dest.column][dest.line].setProperty("hasMoved",true);
                return;
            }
        }

        m_board[dest.column][dest.line] = sourcePiece;
        m_board[source.column][source.line] = Piece();

        if ( move.hasProperty("promotion") )
        {
            m_board[dest.column][dest.line] = Piece(move.color(),move.getProperty("promotionID"));
        }

        // Silly and stupid castling protection
        m_board[dest.column][dest.line].setProperty("hasMoved",true);
    }
    else
    {
        assert(false); // WOOT
    }
}
