#ifndef HISTORYSAVER_HPP
#define HISTORYSAVER_HPP

#include <vector>
#include <string>

#include "Game/Move.hpp"
#include "Game/Boards/ChessBoard.hpp"
#include "History.hpp"

template <typename DataType>
class HistorySaver
{
private:

public:

    /*! Save data in a file
     *  \param fileName the file name where to save the data
     *  \param data the data to save
     *  \return false if the file could not be written
     */
    static bool save(const std::string& fileName, const DataType& data);
};

template <>
class HistorySaver<Move<ChessBoard::Coords> >
{
public:

    /*! Save chess moves in a file
     *  \param fileName the file name where to save the data
     *  \param data the data to save
     *  \warning the incomplete turn will not be save
     *  \return false if the file could not be written
     */
    static bool save(const std::string& fileName, const History<Move<ChessBoard::Coords> >& data);
};

/*! \class HistorySaver HistorySaver.hpp Game/HistorySaver.hpp
 *  \brief Generic HistorySaver
 *
 *  Saves an history in a file
 */

/*! \class HistorySaver<Move<ChessBoard::Coords> > HistorySaver.hpp Game/HistorySaver.hpp
 *  \brief Specific history saver for chess game moves
 *
 *  The file is using this format :
 *  move1 - move2
 * \sa Move
 */


#endif
