#ifndef CHESSRULES_HPP
#define CHESSRULES_HPP

#include "Rules2D.hpp"

#include "Game/Boards/Board2D.hpp"



class Piece;

class ChessRules : public Rules<ChessBoard>, public Rules2D
{
    public:
        // Define pieces' type for the Chess
        
        static const int PT_PAWN = 1;       /*!< */
        static const int PT_ROOK = 2;       /*!< */
        static const int PT_KNIGHT = 3;     /*!< */
        static const int PT_BISHOP = 4;     /*!< */
        static const int PT_QUEEN = 5;      /*!< */
        static const int PT_KING = 6;       /*!< */

        static const int PC_WHITE = 0;      /*!< */
        static const int PC_BLACK = 1;      /*!< */

    private:

        float m_nbTurnsPat;

        bool checkPawnMove(const ChessBoard& cb, Move<ChessBoard::Coords>& move)const;
        bool checkRookMove(const ChessBoard& cb, Move<ChessBoard::Coords>& move)const;
        bool checkKnightMove(const ChessBoard& cb, Move<ChessBoard::Coords>& move)const;
        bool checkBishopMove(const ChessBoard& cb, Move<ChessBoard::Coords>& move)const;
        bool checkQueenMove(const ChessBoard& cb, Move<ChessBoard::Coords>& move)const;
        bool checkKingMove(const ChessBoard& cb, Move<ChessBoard::Coords>& move)const;

        // Special moves
        bool checkCastlingMove(const ChessBoard& cb, Move<ChessBoard::Coords>& move)const;

        bool checkMove(const ChessBoard& cb, Move<ChessBoard::Coords>& move)const;
        bool checkCheck(const ChessBoard& cb, Move<ChessBoard::Coords>& move)const;

        bool isCheck(const ChessBoard& cb, const std::vector<std::pair<ChessBoard::Coords, Piece> >& pieces, const ChessBoard::Coords& kingCoords, const int color)const;

        // const ChessBoard::Coords& getPieceCoord(const std::vector<std::pair<ChessBoard::Coords, Piece> >& pieces, int type, int color);
        const ChessBoard::Coords& getKingCoords(const std::vector<std::pair<ChessBoard::Coords, Piece> >& pieces, int color)const;

    public:
        static const unsigned int NB_PLAYERS = 2;

        void init(ChessBoard& cb);
        bool check(const ChessBoard& cb, Move<ChessBoard::Coords>& move)const;

        bool isCheck(const ChessBoard& cb, const int color)const;
        bool checkVictory(const ChessBoard& cb, const int color)const;

        bool checkDraw(const History<Move<ChessBoard::Coords> >& history)const;
        void move(const ChessBoard& cb, const Move<ChessBoard::Coords>& move);
};

/*! \class ChessRules ChessRules.hpp Game/Rules/ChessRules.hpp
 *  \brief Set of rules for chess
 *
 *  These rules determines if a move is valid or not (thanks to \c check()).
 *  Moreover, the rules can also check if a player if victorious, or if it is a draw,
 *  respectively with \c checkVictory() and \c checkDraw()
 *  To complete the needs of game settings, the function \c init() will define
 *  the set of initial pieces to start the game.
 */

#endif
