#ifndef GAMEFACTORY_HPP
#define GAMEFACTORY_HPP

#include <vector>

class AbstractGame;

class GameFactory
{
public:
    static const int GT_Chess = 0;      /*!< Chess game */

    static const int CT_GNUChess = 0;   /*!< ID for controler played by gnuchess */
    static const int CT_Player = 1;     /*!< player input */

private:

public:
    /*! Gets the number of controlers allowed by the game corresponding to gameType
     *  \param gameType the type of the game to check
     *  \return number of controlers
     */
    static int getNbControlersNeeded(int gameType);

    /*! Create a new game
     *  \param gameType the type of the game to create
     *  \param controlerTypes list of controler ids to put in the game
     *  \return the game as an AbstractGame
     *  \warning the pointer returned should be free
     *  \todo the pointer can be replaced by a smart pointer
     */
    static AbstractGame* createGame(int gameType, const std::vector<int>& controlerTypes);
};

/*! \class GameFactory GameFactory.hpp GameFactory.hpp
 *  \brief Constructs a new game following settings
 *
 *  The factory brings ID to recognise the game type (GT_*) et the controlers type (CT_*)
 *  From these ids, we can construct the corresponding game, which will be returned as
 *  an AbstractGame.
 */

#endif
