#ifndef CONSOLEDISPLAY_HPP
#define CONSOLEDISPLAY_HPP

#include "Renderer2D.hpp"

#include "Game/Boards/ChessBoard.hpp"
#include "Game/Move.hpp"

#include <iostream>

class ConsoleChessDisplay : public Renderer2D<ChessBoard>
{
private:

public:

    /*! Causes a complete redraw of the game
     */
    void notify(const Move<ChessBoard::Coords> & move)
    {
        (void)move;

        displayBoard();
    }

    virtual void displayBoard()
    {
        for ( int x = Renderer<ChessBoard>::m_pBoard->getDimensionLength(1)-1 ; x >= 0 ; x-- )
        {
            for ( int y = 0 ; y < Renderer<ChessBoard>::m_pBoard->getDimensionLength(0) ; y++ )
            {
                displayPiece(ChessBoard::Coords(y,x));
                std::cout << " ";
            }
            std::cout << std::endl;
        }
    }

    void displayPiece(const ChessBoard::Coords& position)
    {
        const Piece& piece = m_pBoard->get(position.get());
        char pieceDisp = ' ';
        switch (piece.type())
        {
            case 1:
                pieceDisp = 'p';
                break;
            case 2:
                pieceDisp = 'r';
                break;
            case 3:
                pieceDisp = 'n';
                break;
            case 4:
                pieceDisp = 'b';
                break;
            case 5:
                pieceDisp = 'q';
                break;
            case 6:
                pieceDisp = 'k';
                break;
            case 0:
            default:
                pieceDisp = '.';
                break;
        }

        if ( piece.color() == 0 )
        {
            pieceDisp = toupper(pieceDisp);
        }

        std::cout << pieceDisp;
    }
};

/*! \class ConsoleChessDisplay ConsoleChessDisplay.hpp Renderers/ConsoleChessDisplay.hpp
 *  \brief Renderer outputting on console
 *
 *  The output is looking similar to GNUChess console output.
 *  The following letters are used for the pieces:
 *  - p : pawn
 *  - r : rook
 *  - n : knight
 *  - b : bishop
 *  - q : queen
 *  - k : king
 *
 *  The white are in upper case, black in lower case
 *  Whites are always displayed at the bottom
 */

#endif
