#ifndef MOVEPARSER_HPP
#define MOVEPARSER_HPP

#include <string>
#include <iostream>

#include "Game/Move.hpp"
#include "Game/Boards/ChessBoard.hpp"

class MoveParser
{
private:
    static void deleteSpaces(std::string& str);
public:

    // template <typename Coords>
    /*! Extracts a Move for Chessboard from a string
     *  In case of failure during the parsing process, an invalid move is returned. The
     *  validity can be checkes using isLegal()
     *  \param input the string to parse
     *  \param moveColor the player color initiating the move
     *  \return the move corresponding to input or an invalid move if the parsing failed
     */
    static Move<ChessBoard::Coords> parse(const std::string& input, int moveColor)
    {
        std::string buffer = input;
        std::vector<ChessBoard::Coords> coords;

        deleteSpaces(buffer);

        if ( buffer.length() >= 4 ) // 4 normal move
        {
            for ( unsigned int i = 0 ; i < 4 ; i+=2 )
            {
                int columnNumber = tolower(buffer[i])-'a';
                int lineNumber = tolower(buffer[i+1])-'0';

                ChessBoard::Coords newCoords(columnNumber,lineNumber-1);
                if ( !newCoords.isValid() )
                {
                    std::cout << "Try to parse this move :'" << buffer << "'" << std::endl;
                    return Move<ChessBoard::Coords>();
                }
                else
                {
                    coords.push_back(newCoords);
                }
            }

            // What is after, we don't mind
        }
        else
        {
            std::cout << "Try to parse this move :'" << buffer << "'" << std::endl;
            return Move<ChessBoard::Coords>();
        }

        Move<ChessBoard::Coords> move(moveColor,coords[0],coords[1]);

        if ( buffer.length() == 5 ) // Promotion
        {
            if ( buffer[4] != '\n' && buffer[4] != '\r' && buffer[4] != ' ' ) // Since Windows sucks !
            {
                move.setProperty("promotion",buffer[4]);
            }
        }
        return move;
    }

};

/*! \class MoveParser MoveParser.hpp MoveParser.hpp
 *  \brief Utility class to create a Move from a string
 *
 *  The move can have extra spaces, since they are removed before
 *  parsing.
 *  A move is characterised by two coordinates (in chess notation, e.g.
 *  a letter between 'a' and 'h' and a number between 1 and 8.
 *  A fifth letter reprentig a piece can be added, for promition.
 *  The accepted letters are : 'q' for queen, 'r' for rook, 'b' for
 *  bishop and 'n' for knight.
 */


#endif
