#ifndef NOTIFIER_HPP
#define NOTIFIER_HPP

#include "Listener.hpp"

#include <vector>
#include <cassert>


template <typename MessageType>
class Notifier
{
private:

    std::vector<Listener<MessageType>*> m_listeners;    /*!< List of the listener to prevent */

public:

    /*! Adds a listener to the list of instance to call
     *  \param pListener new listener
     */
    void addListener(Listener<MessageType>* pListener)
    {
        assert(pListener);
        m_listeners.push_back(pListener);
    }

    /*! Notify all listeners of the event
     *  \param message the associated message to transfert to the listeners
     */
    void notify(const MessageType& message)
    {
        // TODO replace the loop
        // std::for_each(m_listeners.begin(),m_listeners.end(),)
        for ( typename std::vector<Listener<MessageType>*>::const_iterator itListener = m_listeners.begin() ; itListener != m_listeners.end() ; ++itListener )
        {
            (*itListener)->notify(message);
        }
    }

/*! \class Notifier Notifier.hpp Utils/Notifier.hpp
 *  \brief Notifier base for notifier/listener design pattern
 *
 *  The notifier will prevent all registered listener of a new message
 */

};

#endif
