#include "tst_bishopmoves.hpp"

#include <QtCore/QString>
#include <QtTest/QtTest>

#include <cmath>

BishopMovesTest::BishopMovesTest()
{
}

void BishopMovesTest::initTestCase()
{
    sourceCoords.column = 4;
    sourceCoords.line = 4;

    std::vector<std::pair<ChessBoard::Coords, Piece> > initialPieces;
    initialPieces.push_back(std::pair<ChessBoard::Coords, Piece>(ChessBoard::Coords(0,2), Piece(ChessRules::PC_BLACK,ChessRules::PT_KING)));
    initialPieces.push_back(std::pair<ChessBoard::Coords, Piece>(ChessBoard::Coords(7,5), Piece(ChessRules::PC_WHITE,ChessRules::PT_KING)));
    initialPieces.push_back(std::pair<ChessBoard::Coords, Piece>(sourceCoords, Piece(ChessRules::PC_BLACK,ChessRules::PT_BISHOP)));

    m_board.init(initialPieces);
}

void BishopMovesTest::cleanupTestCase()
{
}

void BishopMovesTest::bishopTesting()
{
    QFETCH(int, columnDest);
    QFETCH(int, lineDest);
    QFETCH(bool, result);

    Move<ChessBoard::Coords> m(ChessRules::PC_BLACK, sourceCoords, ChessBoard::Coords(columnDest,lineDest));
    QCOMPARE(m_rules.check(m_board,m),result);
}

void BishopMovesTest::bishopTesting_data()
{
    QTest::addColumn<int>("columnDest");
    QTest::addColumn<int>("lineDest");
    QTest::addColumn<bool>("result");

    for ( int column = 0 ; column < m_board.getDimensionLength(0) ; column++ )
    {
        for ( int line = 0 ; line < m_board.getDimensionLength(1) ; line++ )
        {
            bool result = false;
            int distX = std::abs(static_cast<float>(sourceCoords.column - column));
            int distY = std::abs(static_cast<float>(sourceCoords.line - line));

            if ( distX == distY )
            {
                if ( distX != 0 )
                {
                    result = true;
                }
            }

            QTest::newRow((QString("Destination ") +
                           QString::number(column) +
                           QString(";") +
                           QString::number(line)).toStdString().c_str()) <<
                                                                            column <<
                                                                            line <<
                                                                            result;
        }
    }
}
