#include "tst_checkmate.hpp"

#include <QtCore/QString>
#include <QtTest/QtTest>

#include <cmath>

CheckMateTest::CheckMateTest()
{
}

void CheckMateTest::initTestCase()
{
    {
        std::vector<std::pair<ChessBoard::Coords, Piece> > initialPieces;
        initialPieces.push_back(std::pair<ChessBoard::Coords, Piece>(ChessBoard::Coords(0,4), Piece(ChessRules::PC_BLACK,ChessRules::PT_KING)));
        initialPieces.push_back(std::pair<ChessBoard::Coords, Piece>(ChessBoard::Coords(7,5), Piece(ChessRules::PC_WHITE,ChessRules::PT_KING)));
        initialPieces.push_back(std::pair<ChessBoard::Coords, Piece>(ChessBoard::Coords(0,0), Piece(ChessRules::PC_WHITE,ChessRules::PT_ROOK)));
        initialPieces.push_back(std::pair<ChessBoard::Coords, Piece>(ChessBoard::Coords(1,0), Piece(ChessRules::PC_WHITE,ChessRules::PT_QUEEN)));

        m_boardTest1.init(initialPieces);
    }

    {
        std::vector<std::pair<ChessBoard::Coords, Piece> > initialPieces;
        initialPieces.push_back(std::pair<ChessBoard::Coords, Piece>(ChessBoard::Coords(0,4), Piece(ChessRules::PC_BLACK,ChessRules::PT_KING)));
        initialPieces.push_back(std::pair<ChessBoard::Coords, Piece>(ChessBoard::Coords(2,5), Piece(ChessRules::PC_BLACK,ChessRules::PT_PAWN))); // Useless non protecting piece

        initialPieces.push_back(std::pair<ChessBoard::Coords, Piece>(ChessBoard::Coords(7,5), Piece(ChessRules::PC_WHITE,ChessRules::PT_KING)));
        initialPieces.push_back(std::pair<ChessBoard::Coords, Piece>(ChessBoard::Coords(0,0), Piece(ChessRules::PC_WHITE,ChessRules::PT_ROOK)));
        initialPieces.push_back(std::pair<ChessBoard::Coords, Piece>(ChessBoard::Coords(1,0), Piece(ChessRules::PC_WHITE,ChessRules::PT_QUEEN)));

        m_boardTest2.init(initialPieces);
    }

    {
        std::vector<std::pair<ChessBoard::Coords, Piece> > initialPieces;
        initialPieces.push_back(std::pair<ChessBoard::Coords, Piece>(ChessBoard::Coords(0,4), Piece(ChessRules::PC_BLACK,ChessRules::PT_KING)));
        initialPieces.push_back(std::pair<ChessBoard::Coords, Piece>(ChessBoard::Coords(5,2), Piece(ChessRules::PC_BLACK,ChessRules::PT_ROOK))); // Protecting piece

        initialPieces.push_back(std::pair<ChessBoard::Coords, Piece>(ChessBoard::Coords(7,5), Piece(ChessRules::PC_WHITE,ChessRules::PT_KING)));
        initialPieces.push_back(std::pair<ChessBoard::Coords, Piece>(ChessBoard::Coords(0,0), Piece(ChessRules::PC_WHITE,ChessRules::PT_ROOK)));
        initialPieces.push_back(std::pair<ChessBoard::Coords, Piece>(ChessBoard::Coords(1,0), Piece(ChessRules::PC_WHITE,ChessRules::PT_QUEEN)));

        m_boardTest3.init(initialPieces);
    }
}

void CheckMateTest::cleanupTestCase()
{
}

void CheckMateTest::checkMateTesting1()
{
    QCOMPARE(m_rules.checkVictory(m_boardTest1,ChessRules::PC_WHITE),true);
}

void CheckMateTest::checkMateTesting2()
{
    QCOMPARE(m_rules.checkVictory(m_boardTest2,ChessRules::PC_WHITE),true);
}

void CheckMateTest::checkMateProtectionTesting()
{
    QCOMPARE(m_rules.checkVictory(m_boardTest3,ChessRules::PC_WHITE),false);
}
